-- <File header>
-- Project
--    pAVR (pipelined AVR) is an 8 bit RISC controller, compatible with Atmel's
--    AVR core, but about 3x faster in terms of both clock frequency and MIPS.
--    The increase in speed comes from a relatively deep pipeline. The original
--    AVR core has only two pipeline stages (fetch and execute), while pAVR has
--    6 pipeline stages:
--       1. PM    (read Program Memory)
--       2. INSTR (load Instruction)
--       3. RFRD  (decode Instruction and read Register File)
--       4. OPS   (load Operands)
--       5. ALU   (execute ALU opcode or access Unified Memory)
--       6. RFWR  (write Register File)
-- Version
--    0.35
-- Date
--    2004 March 11
-- Author
--    Doru Cuturela, doruu@yahoo.com, geocities.com/doruu
-- License
--    This program is free software; you can redistribute it and/or modify
--    it under the terms of the GNU General Public License as published by
--    the Free Software Foundation; either version 2 of the License, or
--    (at your option) any later version.
--    This program is distributed in the hope that it will be useful,
--    but WITHOUT ANY WARRANTY; without even the implied warranty of
--    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--    GNU General Public License for more details.
--    You should have received a copy of the GNU General Public License
--    along with this program; if not, write to the Free Software
--    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-- </File header>



-- <File info>
-- This is the main test entity. It embedds a pAVR controller, its Program
--    Memory and a muxer for taking/getting Program Memory access to/from pAVR.
-- This entity implements the following behavior:
--    - 1. reset all registers.
--       For the easyness of debugging, they are resetted to a particular non-zero
--       value (0x77).
--       Funny enough, the registers are initialized by pAVR itself, by providing
--       to it code for that.
--    - 2. load the Program Memory with pAVR's program.
--       This section is tagged. By using the TagScan utility, this architecture
--       is automatically modified so that an external binary file is transposed
--       into VHDL statements that load the Program Memory.
--       This section rewrites the code from the previous section (register
--       loading).
--    - 3. Finally, release the reset lines and let pAVR do its job.
-- Note
--    Care has NOT been taken not to generate latches all over these tests.
--    They ARE generated, there are even 3 state latches, for the ease of testing.
--    However, care HAS BEEN taken in pAVR sources that no latches should be
--    generated there. That is because pAVR is meant to be synthesizable, while
--    pAVR tests are not.
-- To do
--    - Afer resetting registers, but before loading binary file, pAVR executes
--       instructions that are made of Xs. It happens that the instruction decoder
--       decodes these into nops, so it's OK, but not nice. This must be corrected.
-- </File info>



-- <File body>
library ieee;
use ieee.std_logic_1164.all;
library work;
use work.std_util.all;
use work.pavr_util.all;
use work.pavr_constants.all;
use work.test_pavr_util.all;
use work.test_pavr_constants.all;


entity test_pavr is
end;


architecture test_pavr_arch of test_pavr is
   signal clk, res, syncres: std_logic;

   -- Clock counter
   -- Maximum number of testing clocks: 2 billions instructions. Should be enough...
   signal main_clk_cnt, run_clk_cnt: std_logic_vector(30 downto 0);

   -- Instruction counter
   signal instr_cnt: std_logic_vector(30 downto 0);

   -- Signals to control the mixture formed by pAVR, Program Memory and muxers.
   signal pm_sel:  std_logic;
   signal pm_di:   std_logic_vector(15 downto 0);
   signal pm_wr:   std_logic;
   signal pm_addr: std_logic_vector(21 downto 0);
   signal pa:      std_logic_vector(7 downto 0);

   -- pAVR controller-related connectivity
   signal pavr_pavr_res:     std_logic;
   signal pavr_pavr_syncres: std_logic;
   signal pavr_pavr_pm_addr: std_logic_vector(21 downto 0);
   signal pavr_pavr_pm_do:   std_logic_vector(15 downto 0);
   signal pavr_pavr_pm_wr:   std_logic;
   signal pavr_pavr_inc_instr_cnt: std_logic_vector(1 downto 0);

   -- PM-related connectivity
   signal pm_pavr_pm_wr:   std_logic;
   signal pm_pavr_pm_addr: std_logic_vector(21 downto 0);
   signal pm_pavr_pm_di:   std_logic_vector(15 downto 0);
   signal pm_pavr_pm_do:   std_logic_vector(15 downto 0);

   -- Declare the pAVR controller.
   component pavr
   port(
      pavr_clk:      in    std_logic;
      pavr_res:      in    std_logic;
      pavr_syncres:  in    std_logic;
      pavr_pm_addr:  out   std_logic_vector(21 downto 0);
      pavr_pm_do:    in    std_logic_vector(15 downto 0);
      pavr_pm_wr:    out   std_logic;
      pavr_pa:       inout std_logic_vector(7 downto 0);
      pavr_inc_instr_cnt: out std_logic_vector(1 downto 0)
   );
   end component;
   for all: pavr use entity work.pavr(pavr_arch);

   -- Declare the Program Memory.
   component pavr_pm
   port(
      pavr_pm_clk:  in  std_logic;
      pavr_pm_wr:   in  std_logic;
      pavr_pm_addr: in  std_logic_vector(21 downto 0);
      pavr_pm_di:   in  std_logic_vector(15 downto 0);
      pavr_pm_do:   out std_logic_vector(15 downto 0)
   );
   end component;
   for all: pavr_pm use entity work.pavr_pm(pavr_pm_arch);

begin

   -- Instantiate the pAVR controller.
   pavr_instance1: pavr
   port map(
      clk,
      pavr_pavr_res,
      pavr_pavr_syncres,
      pavr_pavr_pm_addr,
      pavr_pavr_pm_do,
      pavr_pavr_pm_wr,
      pa,
      pavr_pavr_inc_instr_cnt
   );

   -- Instantiate the Program Memory.
   pavr_pm_instance1: pavr_pm
   port map(
      clk,
      pm_pavr_pm_wr,
      pm_pavr_pm_addr,
      pm_pavr_pm_di,
      pm_pavr_pm_do
   );

   generate_clock:
   process
   begin
      clk <= '1';
      wait for 50 ns;
      clk <= '0';
      wait for 50 ns;
   end process generate_clock;


   generate_reset:
   process
   begin
      --res <= '0';
      --wait for 100 ns;
      res <= '1';
      wait for 110 ns;
      res <= '0';
      wait for 1000000 sec;
   end process generate_reset;


   generate_sync_reset:
   process
   begin
      --syncres <= '0';
      --wait for 300 ns;
      syncres <= '1';
      wait for 310 ns;
      syncres <= '0';
      wait for 1000000 sec;
   end process generate_sync_reset;


   test_main:
   process(clk, res, syncres,
           pm_di, pm_addr, pavr_pavr_res, pavr_pavr_syncres,
           main_clk_cnt, run_clk_cnt, instr_cnt,
           pavr_pavr_inc_instr_cnt
          )
      variable tmpv1: std_logic_vector(pm_di'length+pm_addr'length-1 downto 0);  -- This will generate a latch, but who cares? this is not intended to be synthesized.
   begin
      if res='1' then
         -- Async reset

         -- Reset the controller in turn.
         pavr_pavr_res <= '1';

         main_clk_cnt <= int_to_std_logic_vector(0, main_clk_cnt'length);
         run_clk_cnt  <= int_to_std_logic_vector(0, run_clk_cnt'length);
         instr_cnt    <= int_to_std_logic_vector(0, instr_cnt'length);
      elsif clk'event and clk='1' then
         -- Clock counter
         main_clk_cnt <= main_clk_cnt+1;
         run_clk_cnt  <= run_clk_cnt+1;

         -- Instruction counter
         case pavr_pavr_inc_instr_cnt is
            when "01" =>
               instr_cnt <= instr_cnt+1;
            when "10" =>
               instr_cnt <= instr_cnt+2;
            when "11" =>
               instr_cnt <= instr_cnt-1;
            when others =>
               null;
         end case;

         -- Set up Program Memory and let pAVR do its job.
         if (std_logic_vector_to_nat(main_clk_cnt)>=3 and std_logic_vector_to_nat(main_clk_cnt)<35) or
            (std_logic_vector_to_nat(main_clk_cnt)>=100 and std_logic_vector_to_nat(main_clk_cnt)<
         -- The following tagged lines are written automatically, based on the binary file,
         --    by using the tagScan utility.
-- <Clk Cnt>
539
-- </Clk Cnt>
            ) then
            -- Set up Program Memory.
            case std_logic_vector_to_nat(main_clk_cnt) is
               -- Initialize all 32 registers to 0x77.
               when  3 => tmpv1 := pm_setup( 0, 16#e747#);
               when  4 => tmpv1 := pm_setup( 1, 16#2e04#);
               when  5 => tmpv1 := pm_setup( 2, 16#2e14#);
               when  6 => tmpv1 := pm_setup( 3, 16#2e24#);
               when  7 => tmpv1 := pm_setup( 4, 16#2e34#);
               when  8 => tmpv1 := pm_setup( 5, 16#2e44#);
               when  9 => tmpv1 := pm_setup( 6, 16#2e54#);
               when 10 => tmpv1 := pm_setup( 7, 16#2e64#);
               when 11 => tmpv1 := pm_setup( 8, 16#2e74#);
               when 12 => tmpv1 := pm_setup( 9, 16#2e84#);
               when 13 => tmpv1 := pm_setup(10, 16#2e94#);
               when 14 => tmpv1 := pm_setup(11, 16#2ea4#);
               when 15 => tmpv1 := pm_setup(12, 16#2eb4#);
               when 16 => tmpv1 := pm_setup(13, 16#2ec4#);
               when 17 => tmpv1 := pm_setup(14, 16#2ed4#);
               when 18 => tmpv1 := pm_setup(15, 16#2ee4#);
               when 19 => tmpv1 := pm_setup(16, 16#2ef4#);
               when 20 => tmpv1 := pm_setup(17, 16#2f04#);
               when 21 => tmpv1 := pm_setup(18, 16#2f14#);
               when 22 => tmpv1 := pm_setup(19, 16#2f24#);
               when 23 => tmpv1 := pm_setup(20, 16#2f34#);
               when 24 => tmpv1 := pm_setup(21, 16#2f54#);
               when 25 => tmpv1 := pm_setup(22, 16#2f64#);
               when 26 => tmpv1 := pm_setup(23, 16#2f74#);
               when 27 => tmpv1 := pm_setup(24, 16#2f84#);
               when 28 => tmpv1 := pm_setup(25, 16#2f94#);
               when 29 => tmpv1 := pm_setup(26, 16#2fa4#);
               when 30 => tmpv1 := pm_setup(27, 16#2fb4#);
               when 31 => tmpv1 := pm_setup(28, 16#2fc4#);
               when 32 => tmpv1 := pm_setup(29, 16#2fd4#);
               when 33 => tmpv1 := pm_setup(30, 16#2fe4#);
               when 34 => tmpv1 := pm_setup(31, 16#2ff4#);
               -- The following tagged lines are written automatically, based on the binary file,
               --    by using the tagScan utility.
-- <Instructions>
               when 100 => tmpv1 := pm_setup(0, 16#0fc1#);
               when 101 => tmpv1 := pm_setup(1, 16#e910#);
               when 102 => tmpv1 := pm_setup(2, 16#2e01#);
               when 103 => tmpv1 := pm_setup(3, 16#2e11#);
               when 104 => tmpv1 := pm_setup(4, 16#2e21#);
               when 105 => tmpv1 := pm_setup(5, 16#2e41#);
               when 106 => tmpv1 := pm_setup(6, 16#2e71#);
               when 107 => tmpv1 := pm_setup(7, 16#2ec1#);
               when 108 => tmpv1 := pm_setup(8, 16#2ee1#);
               when 109 => tmpv1 := pm_setup(9, 16#2f01#);
               when 110 => tmpv1 := pm_setup(10, 16#2f21#);
               when 111 => tmpv1 := pm_setup(11, 16#2f41#);
               when 112 => tmpv1 := pm_setup(12, 16#2f61#);
               when 113 => tmpv1 := pm_setup(13, 16#2f71#);
               when 114 => tmpv1 := pm_setup(14, 16#2f81#);
               when 115 => tmpv1 := pm_setup(15, 16#2f91#);
               when 116 => tmpv1 := pm_setup(16, 16#2fa1#);
               when 117 => tmpv1 := pm_setup(17, 16#2fd1#);
               when 118 => tmpv1 := pm_setup(18, 16#2ff1#);
               when 119 => tmpv1 := pm_setup(19, 16#edc5#);
               when 120 => tmpv1 := pm_setup(20, 16#2e3c#);
               when 121 => tmpv1 := pm_setup(21, 16#2e5c#);
               when 122 => tmpv1 := pm_setup(22, 16#2e6c#);
               when 123 => tmpv1 := pm_setup(23, 16#2e8c#);
               when 124 => tmpv1 := pm_setup(24, 16#2e9c#);
               when 125 => tmpv1 := pm_setup(25, 16#2eac#);
               when 126 => tmpv1 := pm_setup(26, 16#2ebc#);
               when 127 => tmpv1 := pm_setup(27, 16#2edc#);
               when 128 => tmpv1 := pm_setup(28, 16#2efc#);
               when 129 => tmpv1 := pm_setup(29, 16#2f3c#);
               when 130 => tmpv1 := pm_setup(30, 16#2f5c#);
               when 131 => tmpv1 := pm_setup(31, 16#2f8c#);
               when 132 => tmpv1 := pm_setup(32, 16#2fbc#);
               when 133 => tmpv1 := pm_setup(33, 16#2fec#);
               when 134 => tmpv1 := pm_setup(34, 16#2c00#);
               when 135 => tmpv1 := pm_setup(35, 16#2c00#);
               when 136 => tmpv1 := pm_setup(36, 16#2c00#);
               when 137 => tmpv1 := pm_setup(37, 16#2c00#);
               when 138 => tmpv1 := pm_setup(38, 16#2c00#);
               when 139 => tmpv1 := pm_setup(39, 16#2c00#);
               when 140 => tmpv1 := pm_setup(40, 16#0fc1#);
               when 141 => tmpv1 := pm_setup(41, 16#1f9c#);
               when 142 => tmpv1 := pm_setup(42, 16#96c5#);
               when 143 => tmpv1 := pm_setup(43, 16#1b78#);
               when 144 => tmpv1 := pm_setup(44, 16#5f76#);
               when 145 => tmpv1 := pm_setup(45, 16#0ba7#);
               when 146 => tmpv1 := pm_setup(46, 16#979d#);
               when 147 => tmpv1 := pm_setup(47, 16#95a3#);
               when 148 => tmpv1 := pm_setup(48, 16#95a3#);
               when 149 => tmpv1 := pm_setup(49, 16#95a3#);
               when 150 => tmpv1 := pm_setup(50, 16#95a3#);
               when 151 => tmpv1 := pm_setup(51, 16#95a3#);
               when 152 => tmpv1 := pm_setup(52, 16#95aa#);
               when 153 => tmpv1 := pm_setup(53, 16#23ca#);
               when 154 => tmpv1 := pm_setup(54, 16#74c3#);
               when 155 => tmpv1 := pm_setup(55, 16#2bfc#);
               when 156 => tmpv1 := pm_setup(56, 16#66f3#);
               when 157 => tmpv1 := pm_setup(57, 16#26af#);
               when 158 => tmpv1 := pm_setup(58, 16#94a0#);
               when 159 => tmpv1 := pm_setup(59, 16#94a1#);
               when 160 => tmpv1 := pm_setup(60, 16#9418#);
               when 161 => tmpv1 := pm_setup(61, 16#14ab#);
               when 162 => tmpv1 := pm_setup(62, 16#2cca#);
               when 163 => tmpv1 := pm_setup(63, 16#94ca#);
               when 164 => tmpv1 := pm_setup(64, 16#9418#);
               when 165 => tmpv1 := pm_setup(65, 16#04ac#);
               when 166 => tmpv1 := pm_setup(66, 16#3a1b#);
               when 167 => tmpv1 := pm_setup(67, 16#94c2#);
               when 168 => tmpv1 := pm_setup(68, 16#94c6#);
               when 169 => tmpv1 := pm_setup(69, 16#94c7#);
               when 170 => tmpv1 := pm_setup(70, 16#94c5#);
               when 171 => tmpv1 := pm_setup(71, 16#9caa#);
               when 172 => tmpv1 := pm_setup(72, 16#0211#);
               when 173 => tmpv1 := pm_setup(73, 16#0311#);
               when 174 => tmpv1 := pm_setup(74, 16#0319#);
               when 175 => tmpv1 := pm_setup(75, 16#0391#);
               when 176 => tmpv1 := pm_setup(76, 16#0399#);
               when 177 => tmpv1 := pm_setup(77, 16#2c0c#);
               when 178 => tmpv1 := pm_setup(78, 16#2c1b#);
               when 179 => tmpv1 := pm_setup(79, 16#1810#);
               when 180 => tmpv1 := pm_setup(80, 16#be1f#);
               when 181 => tmpv1 := pm_setup(81, 16#94f8#);
               when 182 => tmpv1 := pm_setup(82, 16#94b8#);
               when 183 => tmpv1 := pm_setup(83, 16#9418#);
               when 184 => tmpv1 := pm_setup(84, 16#9448#);
               when 185 => tmpv1 := pm_setup(85, 16#efbf#);
               when 186 => tmpv1 := pm_setup(86, 16#bbba#);
               when 187 => tmpv1 := pm_setup(87, 16#b7df#);
               when 188 => tmpv1 := pm_setup(88, 16#bbdb#);
               when 189 => tmpv1 := pm_setup(89, 16#98d8#);
               when 190 => tmpv1 := pm_setup(90, 16#98de#);
               when 191 => tmpv1 := pm_setup(91, 16#9adb#);
               when 192 => tmpv1 := pm_setup(92, 16#9add#);
               when 193 => tmpv1 := pm_setup(93, 16#b3eb#);
               when 194 => tmpv1 := pm_setup(94, 16#e0f0#);
               when 195 => tmpv1 := pm_setup(95, 16#97fe#);
               when 196 => tmpv1 := pm_setup(96, 16#fbe1#);
               when 197 => tmpv1 := pm_setup(97, 16#f9e6#);
               when 198 => tmpv1 := pm_setup(98, 16#2df1#);
               when 199 => tmpv1 := pm_setup(99, 16#96ba#);
               when 200 => tmpv1 := pm_setup(100, 16#9678#);
               when 201 => tmpv1 := pm_setup(101, 16#2eae#);
               when 202 => tmpv1 := pm_setup(102, 16#e091#);
               when 203 => tmpv1 := pm_setup(103, 16#bf9e#);
               when 204 => tmpv1 := pm_setup(104, 16#e1ac#);
               when 205 => tmpv1 := pm_setup(105, 16#bfad#);
               when 206 => tmpv1 := pm_setup(106, 16#01bf#);
               when 207 => tmpv1 := pm_setup(107, 16#936f#);
               when 208 => tmpv1 := pm_setup(108, 16#915f#);
               when 209 => tmpv1 := pm_setup(109, 16#0d5c#);
               when 210 => tmpv1 := pm_setup(110, 16#0d5c#);
               when 211 => tmpv1 := pm_setup(111, 16#0d5c#);
               when 212 => tmpv1 := pm_setup(112, 16#0d5c#);
               when 213 => tmpv1 := pm_setup(113, 16#9350#);
               when 214 => tmpv1 := pm_setup(114, 16#0130#);
               when 215 => tmpv1 := pm_setup(115, 16#9160#);
               when 216 => tmpv1 := pm_setup(116, 16#0130#);
               when 217 => tmpv1 := pm_setup(117, 16#e0f1#);
               when 218 => tmpv1 := pm_setup(118, 16#e2e3#);
               when 219 => tmpv1 := pm_setup(119, 16#9563#);
               when 220 => tmpv1 := pm_setup(120, 16#8360#);
               when 221 => tmpv1 := pm_setup(121, 16#8150#);
               when 222 => tmpv1 := pm_setup(122, 16#ef19#);
               when 223 => tmpv1 := pm_setup(123, 16#2e01#);
               when 224 => tmpv1 := pm_setup(124, 16#9403#);
               when 225 => tmpv1 := pm_setup(125, 16#e0b1#);
               when 226 => tmpv1 := pm_setup(126, 16#e4a0#);
               when 227 => tmpv1 := pm_setup(127, 16#e0d1#);
               when 228 => tmpv1 := pm_setup(128, 16#e5c0#);
               when 229 => tmpv1 := pm_setup(129, 16#e0f1#);
               when 230 => tmpv1 := pm_setup(130, 16#e6e0#);
               when 231 => tmpv1 := pm_setup(131, 16#920d#);
               when 232 => tmpv1 := pm_setup(132, 16#920c#);
               when 233 => tmpv1 := pm_setup(133, 16#9612#);
               when 234 => tmpv1 := pm_setup(134, 16#920e#);
               when 235 => tmpv1 := pm_setup(135, 16#914d#);
               when 236 => tmpv1 := pm_setup(136, 16#9712#);
               when 237 => tmpv1 := pm_setup(137, 16#915c#);
               when 238 => tmpv1 := pm_setup(138, 16#916e#);
               when 239 => tmpv1 := pm_setup(139, 16#9403#);
               when 240 => tmpv1 := pm_setup(140, 16#9209#);
               when 241 => tmpv1 := pm_setup(141, 16#8208#);
               when 242 => tmpv1 := pm_setup(142, 16#9622#);
               when 243 => tmpv1 := pm_setup(143, 16#920a#);
               when 244 => tmpv1 := pm_setup(144, 16#9149#);
               when 245 => tmpv1 := pm_setup(145, 16#9722#);
               when 246 => tmpv1 := pm_setup(146, 16#8158#);
               when 247 => tmpv1 := pm_setup(147, 16#916a#);
               when 248 => tmpv1 := pm_setup(148, 16#ae0b#);
               when 249 => tmpv1 := pm_setup(149, 16#ad7b#);
               when 250 => tmpv1 := pm_setup(150, 16#9403#);
               when 251 => tmpv1 := pm_setup(151, 16#9201#);
               when 252 => tmpv1 := pm_setup(152, 16#8200#);
               when 253 => tmpv1 := pm_setup(153, 16#9632#);
               when 254 => tmpv1 := pm_setup(154, 16#9202#);
               when 255 => tmpv1 := pm_setup(155, 16#9141#);
               when 256 => tmpv1 := pm_setup(156, 16#9732#);
               when 257 => tmpv1 := pm_setup(157, 16#8150#);
               when 258 => tmpv1 := pm_setup(158, 16#9162#);
               when 259 => tmpv1 := pm_setup(159, 16#ae04#);
               when 260 => tmpv1 := pm_setup(160, 16#ad74#);
               when 261 => tmpv1 := pm_setup(161, 16#ea4a#);
               when 262 => tmpv1 := pm_setup(162, 16#eb5b#);
               when 263 => tmpv1 := pm_setup(163, 16#e0b0#);
               when 264 => tmpv1 := pm_setup(164, 16#e1a5#);
               when 265 => tmpv1 := pm_setup(165, 16#934d#);
               when 266 => tmpv1 := pm_setup(166, 16#ec4c#);
               when 267 => tmpv1 := pm_setup(167, 16#e0f0#);
               when 268 => tmpv1 := pm_setup(168, 16#e3e8#);
               when 269 => tmpv1 := pm_setup(169, 16#8343#);
               when 270 => tmpv1 := pm_setup(170, 16#e0f3#);
               when 271 => tmpv1 := pm_setup(171, 16#e7e4#);
               when 272 => tmpv1 := pm_setup(172, 16#95c8#);
               when 273 => tmpv1 := pm_setup(173, 16#9115#);
               when 274 => tmpv1 := pm_setup(174, 16#9125#);
               when 275 => tmpv1 := pm_setup(175, 16#9134#);
               when 276 => tmpv1 := pm_setup(176, 16#e0f3#);
               when 277 => tmpv1 := pm_setup(177, 16#e4e5#);
               when 278 => tmpv1 := pm_setup(178, 16#e010#);
               when 279 => tmpv1 := pm_setup(179, 16#bf1b#);
               when 280 => tmpv1 := pm_setup(180, 16#95d8#);
               when 281 => tmpv1 := pm_setup(181, 16#9117#);
               when 282 => tmpv1 := pm_setup(182, 16#9127#);
               when 283 => tmpv1 := pm_setup(183, 16#9136#);
               when 284 => tmpv1 := pm_setup(184, 16#c004#);
               when 285 => tmpv1 := pm_setup(185, 16#ea3a#);
               when 286 => tmpv1 := pm_setup(186, 16#eb3b#);
               when 287 => tmpv1 := pm_setup(187, 16#ec3c#);
               when 288 => tmpv1 := pm_setup(188, 16#c003#);
               when 289 => tmpv1 := pm_setup(189, 16#ed3d#);
               when 290 => tmpv1 := pm_setup(190, 16#ee3e#);
               when 291 => tmpv1 := pm_setup(191, 16#cffa#);
               when 292 => tmpv1 := pm_setup(192, 16#ef3f#);
               when 293 => tmpv1 := pm_setup(193, 16#e131#);
               when 294 => tmpv1 := pm_setup(194, 16#e0f2#);
               when 295 => tmpv1 := pm_setup(195, 16#e5e8#);
               when 296 => tmpv1 := pm_setup(196, 16#9409#);
               when 297 => tmpv1 := pm_setup(197, 16#e232#);
               when 298 => tmpv1 := pm_setup(198, 16#e0f2#);
               when 299 => tmpv1 := pm_setup(199, 16#e5ec#);
               when 300 => tmpv1 := pm_setup(200, 16#9409#);
               when 301 => tmpv1 := pm_setup(201, 16#e333#);
               when 302 => tmpv1 := pm_setup(202, 16#e0f2#);
               when 303 => tmpv1 := pm_setup(203, 16#e5e4#);
               when 304 => tmpv1 := pm_setup(204, 16#9409#);
               when 305 => tmpv1 := pm_setup(205, 16#e434#);
               when 306 => tmpv1 := pm_setup(206, 16#e0f2#);
               when 307 => tmpv1 := pm_setup(207, 16#e6e1#);
               when 308 => tmpv1 := pm_setup(208, 16#9409#);
               when 309 => tmpv1 := pm_setup(209, 16#e535#);
               when 310 => tmpv1 := pm_setup(210, 16#e636#);
               when 311 => tmpv1 := pm_setup(211, 16#e737#);
               when 312 => tmpv1 := pm_setup(212, 16#e010#);
               when 313 => tmpv1 := pm_setup(213, 16#bf1b#);
               when 314 => tmpv1 := pm_setup(214, 16#e0f2#);
               when 315 => tmpv1 := pm_setup(215, 16#e6ea#);
               when 316 => tmpv1 := pm_setup(216, 16#9419#);
               when 317 => tmpv1 := pm_setup(217, 16#e838#);
               when 318 => tmpv1 := pm_setup(218, 16#c004#);
               when 319 => tmpv1 := pm_setup(219, 16#e939#);
               when 320 => tmpv1 := pm_setup(220, 16#e0f2#);
               when 321 => tmpv1 := pm_setup(221, 16#e6e8#);
               when 322 => tmpv1 := pm_setup(222, 16#9419#);
               when 323 => tmpv1 := pm_setup(223, 16#ea3a#);
               when 324 => tmpv1 := pm_setup(224, 16#e130#);
               when 325 => tmpv1 := pm_setup(225, 16#940c#);
               when 326 => tmpv1 := pm_setup(226, 16#00f1#);
               when 327 => tmpv1 := pm_setup(227, 16#e131#);
               when 328 => tmpv1 := pm_setup(228, 16#e132#);
               when 329 => tmpv1 := pm_setup(229, 16#94b3#);
               when 330 => tmpv1 := pm_setup(230, 16#94a3#);
               when 331 => tmpv1 := pm_setup(231, 16#e019#);
               when 332 => tmpv1 := pm_setup(232, 16#ea42#);
               when 333 => tmpv1 := pm_setup(233, 16#e0d0#);
               when 334 => tmpv1 := pm_setup(234, 16#e1c4#);
               when 335 => tmpv1 := pm_setup(235, 16#8318#);
               when 336 => tmpv1 := pm_setup(236, 16#8158#);
               when 337 => tmpv1 := pm_setup(237, 16#940c#);
               when 338 => tmpv1 := pm_setup(238, 16#00f7#);
               when 339 => tmpv1 := pm_setup(239, 16#e133#);
               when 340 => tmpv1 := pm_setup(240, 16#e134#);
               when 341 => tmpv1 := pm_setup(241, 16#e135#);
               when 342 => tmpv1 := pm_setup(242, 16#94b3#);
               when 343 => tmpv1 := pm_setup(243, 16#94a3#);
               when 344 => tmpv1 := pm_setup(244, 16#015c#);
               when 345 => tmpv1 := pm_setup(245, 16#940c#);
               when 346 => tmpv1 := pm_setup(246, 16#00e4#);
               when 347 => tmpv1 := pm_setup(247, 16#e136#);
               when 348 => tmpv1 := pm_setup(248, 16#e140#);
               when 349 => tmpv1 := pm_setup(249, 16#e150#);
               when 350 => tmpv1 := pm_setup(250, 16#e161#);
               when 351 => tmpv1 := pm_setup(251, 16#e231#);
               when 352 => tmpv1 := pm_setup(252, 16#1345#);
               when 353 => tmpv1 := pm_setup(253, 16#e232#);
               when 354 => tmpv1 := pm_setup(254, 16#e233#);
               when 355 => tmpv1 := pm_setup(255, 16#1346#);
               when 356 => tmpv1 := pm_setup(256, 16#e234#);
               when 357 => tmpv1 := pm_setup(257, 16#e0f0#);
               when 358 => tmpv1 := pm_setup(258, 16#e1e3#);
               when 359 => tmpv1 := pm_setup(259, 16#1345#);
               when 360 => tmpv1 := pm_setup(260, 16#9201#);
               when 361 => tmpv1 := pm_setup(261, 16#9381#);
               when 362 => tmpv1 := pm_setup(262, 16#953a#);
               when 363 => tmpv1 := pm_setup(263, 16#1345#);
               when 364 => tmpv1 := pm_setup(264, 16#9062#);
               when 365 => tmpv1 := pm_setup(265, 16#9072#);
               when 366 => tmpv1 := pm_setup(266, 16#1345#);
               when 367 => tmpv1 := pm_setup(267, 16#0192#);
               when 368 => tmpv1 := pm_setup(268, 16#e235#);
               when 369 => tmpv1 := pm_setup(269, 16#1345#);
               when 370 => tmpv1 := pm_setup(270, 16#940c#);
               when 371 => tmpv1 := pm_setup(271, 16#00f7#);
               when 372 => tmpv1 := pm_setup(272, 16#e236#);
               when 373 => tmpv1 := pm_setup(273, 16#1345#);
               when 374 => tmpv1 := pm_setup(274, 16#8370#);
               when 375 => tmpv1 := pm_setup(275, 16#8390#);
               when 376 => tmpv1 := pm_setup(276, 16#e441#);
               when 377 => tmpv1 := pm_setup(277, 16#fd46#);
               when 378 => tmpv1 := pm_setup(278, 16#e330#);
               when 379 => tmpv1 := pm_setup(279, 16#fd47#);
               when 380 => tmpv1 := pm_setup(280, 16#e238#);
               when 381 => tmpv1 := pm_setup(281, 16#e331#);
               when 382 => tmpv1 := pm_setup(282, 16#fd47#);
               when 383 => tmpv1 := pm_setup(283, 16#8120#);
               when 384 => tmpv1 := pm_setup(284, 16#fd47#);
               when 385 => tmpv1 := pm_setup(285, 16#8340#);
               when 386 => tmpv1 := pm_setup(286, 16#e332#);
               when 387 => tmpv1 := pm_setup(287, 16#fd47#);
               when 388 => tmpv1 := pm_setup(288, 16#940c#);
               when 389 => tmpv1 := pm_setup(289, 16#00f7#);
               when 390 => tmpv1 := pm_setup(290, 16#e333#);
               when 391 => tmpv1 := pm_setup(291, 16#fd47#);
               when 392 => tmpv1 := pm_setup(292, 16#0190#);
               when 393 => tmpv1 := pm_setup(293, 16#e33d#);
               when 394 => tmpv1 := pm_setup(294, 16#e942#);
               when 395 => tmpv1 := pm_setup(295, 16#ff46#);
               when 396 => tmpv1 := pm_setup(296, 16#e430#);
               when 397 => tmpv1 := pm_setup(297, 16#ff47#);
               when 398 => tmpv1 := pm_setup(298, 16#e238#);
               when 399 => tmpv1 := pm_setup(299, 16#e431#);
               when 400 => tmpv1 := pm_setup(300, 16#ff47#);
               when 401 => tmpv1 := pm_setup(301, 16#8120#);
               when 402 => tmpv1 := pm_setup(302, 16#ff47#);
               when 403 => tmpv1 := pm_setup(303, 16#8340#);
               when 404 => tmpv1 := pm_setup(304, 16#e432#);
               when 405 => tmpv1 := pm_setup(305, 16#ff47#);
               when 406 => tmpv1 := pm_setup(306, 16#940c#);
               when 407 => tmpv1 := pm_setup(307, 16#00f7#);
               when 408 => tmpv1 := pm_setup(308, 16#e433#);
               when 409 => tmpv1 := pm_setup(309, 16#ff47#);
               when 410 => tmpv1 := pm_setup(310, 16#0190#);
               when 411 => tmpv1 := pm_setup(311, 16#e434#);
               when 412 => tmpv1 := pm_setup(312, 16#e441#);
               when 413 => tmpv1 := pm_setup(313, 16#bb4b#);
               when 414 => tmpv1 := pm_setup(314, 16#99de#);
               when 415 => tmpv1 := pm_setup(315, 16#e530#);
               when 416 => tmpv1 := pm_setup(316, 16#99df#);
               when 417 => tmpv1 := pm_setup(317, 16#e238#);
               when 418 => tmpv1 := pm_setup(318, 16#e531#);
               when 419 => tmpv1 := pm_setup(319, 16#99df#);
               when 420 => tmpv1 := pm_setup(320, 16#8120#);
               when 421 => tmpv1 := pm_setup(321, 16#99df#);
               when 422 => tmpv1 := pm_setup(322, 16#8340#);
               when 423 => tmpv1 := pm_setup(323, 16#e532#);
               when 424 => tmpv1 := pm_setup(324, 16#99df#);
               when 425 => tmpv1 := pm_setup(325, 16#940c#);
               when 426 => tmpv1 := pm_setup(326, 16#00f7#);
               when 427 => tmpv1 := pm_setup(327, 16#e533#);
               when 428 => tmpv1 := pm_setup(328, 16#99df#);
               when 429 => tmpv1 := pm_setup(329, 16#0190#);
               when 430 => tmpv1 := pm_setup(330, 16#e534#);
               when 431 => tmpv1 := pm_setup(331, 16#e942#);
               when 432 => tmpv1 := pm_setup(332, 16#bb4b#);
               when 433 => tmpv1 := pm_setup(333, 16#9bde#);
               when 434 => tmpv1 := pm_setup(334, 16#e630#);
               when 435 => tmpv1 := pm_setup(335, 16#9bdf#);
               when 436 => tmpv1 := pm_setup(336, 16#e238#);
               when 437 => tmpv1 := pm_setup(337, 16#e631#);
               when 438 => tmpv1 := pm_setup(338, 16#9bdf#);
               when 439 => tmpv1 := pm_setup(339, 16#8120#);
               when 440 => tmpv1 := pm_setup(340, 16#9bdf#);
               when 441 => tmpv1 := pm_setup(341, 16#8340#);
               when 442 => tmpv1 := pm_setup(342, 16#e632#);
               when 443 => tmpv1 := pm_setup(343, 16#9bdf#);
               when 444 => tmpv1 := pm_setup(344, 16#940c#);
               when 445 => tmpv1 := pm_setup(345, 16#00f7#);
               when 446 => tmpv1 := pm_setup(346, 16#e633#);
               when 447 => tmpv1 := pm_setup(347, 16#9bdf#);
               when 448 => tmpv1 := pm_setup(348, 16#0190#);
               when 449 => tmpv1 := pm_setup(349, 16#e634#);
               when 450 => tmpv1 := pm_setup(350, 16#e0f1#);
               when 451 => tmpv1 := pm_setup(351, 16#e4ee#);
               when 452 => tmpv1 := pm_setup(352, 16#e405#);
               when 453 => tmpv1 := pm_setup(353, 16#2e30#);
               when 454 => tmpv1 := pm_setup(354, 16#e003#);
               when 455 => tmpv1 := pm_setup(355, 16#2e40#);
               when 456 => tmpv1 := pm_setup(356, 16#e730#);
               when 457 => tmpv1 := pm_setup(357, 16#be4f#);
               when 458 => tmpv1 := pm_setup(358, 16#8235#);
               when 459 => tmpv1 := pm_setup(359, 16#8135#);
               when 460 => tmpv1 := pm_setup(360, 16#f42a#);
               when 461 => tmpv1 := pm_setup(361, 16#e731#);
               when 462 => tmpv1 := pm_setup(362, 16#e732#);
               when 463 => tmpv1 := pm_setup(363, 16#f411#);
               when 464 => tmpv1 := pm_setup(364, 16#e733#);
               when 465 => tmpv1 := pm_setup(365, 16#c007#);
               when 466 => tmpv1 := pm_setup(366, 16#9433#);
               when 467 => tmpv1 := pm_setup(367, 16#be4f#);
               when 468 => tmpv1 := pm_setup(368, 16#8235#);
               when 469 => tmpv1 := pm_setup(369, 16#8135#);
               when 470 => tmpv1 := pm_setup(370, 16#e734#);
               when 471 => tmpv1 := pm_setup(371, 16#f7b3#);
               when 472 => tmpv1 := pm_setup(372, 16#e735#);
               when 473 => tmpv1 := pm_setup(373, 16#e736#);
               when 474 => tmpv1 := pm_setup(374, 16#e830#);
               when 475 => tmpv1 := pm_setup(375, 16#be4f#);
               when 476 => tmpv1 := pm_setup(376, 16#8235#);
               when 477 => tmpv1 := pm_setup(377, 16#8135#);
               when 478 => tmpv1 := pm_setup(378, 16#f029#);
               when 479 => tmpv1 := pm_setup(379, 16#e831#);
               when 480 => tmpv1 := pm_setup(380, 16#e832#);
               when 481 => tmpv1 := pm_setup(381, 16#f384#);
               when 482 => tmpv1 := pm_setup(382, 16#e833#);
               when 483 => tmpv1 := pm_setup(383, 16#c007#);
               when 484 => tmpv1 := pm_setup(384, 16#9433#);
               when 485 => tmpv1 := pm_setup(385, 16#be4f#);
               when 486 => tmpv1 := pm_setup(386, 16#8235#);
               when 487 => tmpv1 := pm_setup(387, 16#8135#);
               when 488 => tmpv1 := pm_setup(388, 16#e834#);
               when 489 => tmpv1 := pm_setup(389, 16#f3b1#);
               when 490 => tmpv1 := pm_setup(390, 16#e835#);
               when 491 => tmpv1 := pm_setup(391, 16#e836#);
               when 492 => tmpv1 := pm_setup(392, 16#d005#);
               when 493 => tmpv1 := pm_setup(393, 16#ea30#);
               when 494 => tmpv1 := pm_setup(394, 16#940e#);
               when 495 => tmpv1 := pm_setup(395, 16#0192#);
               when 496 => tmpv1 := pm_setup(396, 16#940c#);
               when 497 => tmpv1 := pm_setup(397, 16#01ab#);
               when 498 => tmpv1 := pm_setup(398, 16#ea31#);
               when 499 => tmpv1 := pm_setup(399, 16#8235#);
               when 500 => tmpv1 := pm_setup(400, 16#9508#);
               when 501 => tmpv1 := pm_setup(401, 16#8235#);
               when 502 => tmpv1 := pm_setup(402, 16#ea33#);
               when 503 => tmpv1 := pm_setup(403, 16#9508#);
               when 504 => tmpv1 := pm_setup(404, 16#940c#);
               when 505 => tmpv1 := pm_setup(405, 16#0192#);
               when 506 => tmpv1 := pm_setup(406, 16#ea34#);
               when 507 => tmpv1 := pm_setup(407, 16#96c5#);
               when 508 => tmpv1 := pm_setup(408, 16#9508#);
               when 509 => tmpv1 := pm_setup(409, 16#f2a4#);
               when 510 => tmpv1 := pm_setup(410, 16#ea35#);
               when 511 => tmpv1 := pm_setup(411, 16#2400#);
               when 512 => tmpv1 := pm_setup(412, 16#be0f#);
               when 513 => tmpv1 := pm_setup(413, 16#f3e4#);
               when 514 => tmpv1 := pm_setup(414, 16#9508#);
               when 515 => tmpv1 := pm_setup(415, 16#f3d4#);
               when 516 => tmpv1 := pm_setup(416, 16#ea36#);
               when 517 => tmpv1 := pm_setup(417, 16#940c#);
               when 518 => tmpv1 := pm_setup(418, 16#01a3#);
               when 519 => tmpv1 := pm_setup(419, 16#9508#);
               when 520 => tmpv1 := pm_setup(420, 16#940c#);
               when 521 => tmpv1 := pm_setup(421, 16#01a0#);
               when 522 => tmpv1 := pm_setup(422, 16#ea37#);
               when 523 => tmpv1 := pm_setup(423, 16#2711#);
               when 524 => tmpv1 := pm_setup(424, 16#9117#);
               when 525 => tmpv1 := pm_setup(425, 16#9508#);
               when 526 => tmpv1 := pm_setup(426, 16#1345#);
               when 527 => tmpv1 := pm_setup(427, 16#ea38#);
               when 528 => tmpv1 := pm_setup(428, 16#0000#);
               when 529 => tmpv1 := pm_setup(429, 16#0000#);
               when 530 => tmpv1 := pm_setup(430, 16#0000#);
               when 531 => tmpv1 := pm_setup(431, 16#0000#);
               when 532 => tmpv1 := pm_setup(432, 16#0000#);
               when 533 => tmpv1 := pm_setup(433, 16#0000#);
               when 534 => tmpv1 := pm_setup(434, 16#0000#);
               when 535 => tmpv1 := pm_setup(435, 16#0000#);
               when 536 => tmpv1 := pm_setup(436, 16#0000#);
               when 537 => tmpv1 := pm_setup(437, 16#0000#);
               when 538 => tmpv1 := pm_setup(438, 16#cfff#);
-- </Instructions>

               when others =>
                  null;
            end case;
            pm_sel   <= pm_sel_usr;
            pm_wr    <= '1';
            pm_addr  <= tmpv1(tmpv1'length-1 downto 16);
            pm_di    <= tmpv1(15 downto 0);
            -- Keep the controller in reset state until the Program Memory is prepared.
            pavr_pavr_res     <= '1';
            pavr_pavr_syncres <= '1';
         else
            pm_sel   <= pm_sel_pavr;
            pm_wr    <= '0';
            pm_addr  <= int_to_std_logic_vector(0, pm_addr'length);
            pm_di    <= int_to_std_logic_vector(0, pm_di'length);
            -- Now the Program Memory is prepared. Free the beast.
            pavr_pavr_res     <= '0';
            pavr_pavr_syncres <= '0';
         end if;

         if pavr_pavr_res='1' or pavr_pavr_syncres='1' then
            instr_cnt   <= int_to_std_logic_vector(0, instr_cnt'length);
            run_clk_cnt <= int_to_std_logic_vector(0, run_clk_cnt'length);
         end if;

         if syncres='1' then
            -- Sync reset

            -- Reset the controller in turn.
            pavr_pavr_syncres <= '1';

            main_clk_cnt <= int_to_std_logic_vector(0, main_clk_cnt'length);
            run_clk_cnt  <= int_to_std_logic_vector(0, run_clk_cnt'length);
            instr_cnt    <= int_to_std_logic_vector(0, instr_cnt'length);
         end if;
      end if;
   end process test_main;


   -- Connect components.
   select_muxers:
   process(pavr_pavr_pm_addr, pm_addr, pm_sel)
   begin
      if pm_sel=pm_sel_pavr then
         pm_pavr_pm_addr <= pavr_pavr_pm_addr;
      else
         pm_pavr_pm_addr <= pm_addr;
      end if;
   end process select_muxers;
   pm_pavr_pm_wr     <= pm_wr;
   pm_pavr_pm_di     <= pm_di;
   pavr_pavr_pm_do   <= pm_pavr_pm_do;

end;
-- </File body>
